import {DataRectangle, LabeledRect} from '@/types'
import {reactive, Ref} from 'vue'
import {fabric} from 'fabric'
import useStore from "@/store/useStore";

export default (initCanvas?: Ref<fabric.Canvas | undefined>) => {
    const {addToModel, resetModelPage} = useStore()

    const setInternalModel = (currentPage: number, instanceCanvas?: fabric.Canvas | undefined) => {
        const canvas = initCanvas?.value ? initCanvas.value : instanceCanvas
        const rects = canvas?.getObjects('labeledRect') as LabeledRect[]
        if (rects?.length) {
            resetModelPage(currentPage)
            addToModel(currentPage, ...rects.map((rect) => {
                    return reactive<DataRectangle>({
                        angle: rect.angle || 0,
                        color: '' + rect.fill || '',
                        data: rect.data,
                        height: rect.height || 0,
                        isRotatable: rect.isControlVisible('mtr'),
                        label: rect.label,
                        left: rect.left || 0,
                        top: rect.top || 0,
                        width: rect.width || 0,
                        uid: rect.uid,
                    })
                },
            ))
        }
    }
    return {
        setInternalModel,
    }
}
