import {fabric} from 'fabric'
import {BaseObject} from '@/types'
import useThrottle from '@/composables/useThrottle'

export default fabric.util.createClass(fabric.Rect, {
    type: 'labeledRect', // This is a fabric type
    label: '',
    cachedLabel: '',
    vModelEmit: null,
    uid: '',

    initialize: function (options: BaseObject) {
        this.callSuper('initialize', options)
        this.set('label', options.label || '')
        this.cacheProperties.push('label')
        this.initListeners()
    },

    toObject: function () {
        return fabric.util.object.extend(this.callSuper('toObject'), {
            label: this.get('label'),
        })
    },

    _render: function (ctx: CanvasRenderingContext2D) {
        this.callSuper('_render', ctx)
        ctx.font = '10px Helvetica'
        ctx.fillStyle = '#333'
        ctx.textBaseline = 'middle'
        ctx.textAlign = 'center'
        ctx.fillText(this.label, 0, 1)
    },

    set: function (key: string, value: any) {
        this.callSuper('set', key, value)
        if (key === 'label') {
            if (value) {
                this.cachedLabel = value
            }
            this.canvas?.requestRenderAll()
        }
        this.canvas?.fire('object:modified', {target: this})
    },

    restoreLabel: function () {
        this.set('label', this.cachedLabel)
    },

    initListeners() {
        const throttle = useThrottle()
        let isScaling = false
        let isFinishedScaling = false // This is required due to the throttle function

        // Without these, the label stretches when scaling as the scaling is actually a 'zoom' and not a resize
        this.on('scaling', throttle(() => {
            if (isFinishedScaling) {
                return
            }
            isScaling = true
            // const rect = e.target as Rectangle
            if (this.label) {
                this.set('label', '')
            }
        }, 100))

        this.on('modified', () => {
            if (isScaling) {
                isFinishedScaling = true
                isScaling = false
                this.set({
                    width: (this.scaleX || 1) * this.width,
                    height: (this.scaleY || 1) * this.height,
                    scaleX: 1,
                    scaleY: 1,
                })
                this.restoreLabel()
                this.canvas?.requestRenderAll()
                setTimeout(() => {
                    // set this to false so that the next scaling event won't trigger the scaling event again
                    // useTimeout is required because the scaling event is throttled. This delay ensures the scaling
                    // has been completed
                    isFinishedScaling = false
                }, 300)
            }
        })
    },
})
