import {reactive, toRefs} from '@vue/reactivity'
import {DataRectangle} from "@/types";
import {readonly, ref} from "vue";

/**
 * Define our state
 *
 * This state is mutable in the function below, but if we pass it to the component without `toRefs` it will not mutate.
 */
const markerModel = ref([] as DataRectangle[][])
const isDrawing = ref(false)

const resetModelPage = (pageNumber: number) => markerModel.value[pageNumber - 1] = []

const addToModel = (pageNumber: number, ...items: DataRectangle[]) => {
    if (markerModel.value.length < pageNumber) {
        markerModel.value.push([])
    }

    items = items.reverse().filter((drawRectangle, index, self) =>
        index === self.findIndex(rect => rect.uid === drawRectangle.uid)
    ).reverse()

    markerModel.value[pageNumber - 1].push(...items)
}

const setIsDrawing = (value: boolean) => isDrawing.value = value

export default () => {
    /**
     * Return the state as a `reference` to make it mutable.
     */
    return {
        isDrawing: readonly(isDrawing),
        markerModel: readonly(markerModel),
        addToModel,
        resetModelPage,
        setIsDrawing,
    }
}
